"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeBlock = exports.codeBlockType = exports.codeBlockLangAttr = exports.codeBlockNodeName = exports.resetCodeblock = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const keymap_1 = require("../../../utils/keymap");
const CodeBlockSpecs_1 = require("./CodeBlockSpecs");
const commands_1 = require("./commands");
const const_1 = require("./const");
const handle_paste_1 = require("./handle-paste");
var commands_2 = require("./commands");
Object.defineProperty(exports, "resetCodeblock", { enumerable: true, get: function () { return commands_2.resetCodeblock; } });
var CodeBlockSpecs_2 = require("./CodeBlockSpecs");
Object.defineProperty(exports, "codeBlockNodeName", { enumerable: true, get: function () { return CodeBlockSpecs_2.codeBlockNodeName; } });
Object.defineProperty(exports, "codeBlockLangAttr", { enumerable: true, get: function () { return CodeBlockSpecs_2.codeBlockLangAttr; } });
Object.defineProperty(exports, "codeBlockType", { enumerable: true, get: function () { return CodeBlockSpecs_2.codeBlockType; } });
const CodeBlock = (builder, opts) => {
    builder.use(CodeBlockSpecs_1.CodeBlockSpecs, opts);
    builder.addKeymap((deps) => {
        const { codeBlockKey } = opts;
        const bindings = { Backspace: commands_1.resetCodeblock };
        if (codeBlockKey) {
            bindings[codeBlockKey] = (0, keymap_1.withLogAction)('code_block', (0, prosemirror_commands_1.setBlockType)((0, const_1.cbType)(deps.schema)));
        }
        return bindings;
    });
    builder.addInputRules(({ schema }) => ({ rules: [codeBlockRule((0, const_1.cbType)(schema))] }));
    builder.addAction(const_1.cbAction, ({ schema, serializer }) => {
        const cb = (0, const_1.cbType)(schema);
        const cmd = (state, dispatch) => {
            if (!(0, prosemirror_commands_1.setBlockType)(cb)(state))
                return false;
            if (dispatch) {
                const markup = serializer.serialize(prosemirror_model_1.Fragment.from(state.selection.content().content));
                dispatch(state.tr.replaceSelectionWith(cb.createAndFill({}, markup ? state.schema.text(markup) : null)));
            }
            return true;
        };
        return {
            isActive: (state) => (0, prosemirror_utils_1.hasParentNodeOfType)(cb)(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
    builder.addPlugin(() => new prosemirror_state_1.Plugin({
        props: {
            handleDOMEvents: {
                paste(view, e) {
                    if ((0, handle_paste_1.handlePaste)(view, e, prosemirror_model_1.Slice.empty)) {
                        e.preventDefault();
                        return true;
                    }
                    return false;
                },
            },
        },
    }), builder.Priority.High);
};
exports.CodeBlock = CodeBlock;
// Given a code block node type, returns an input rule that turns a
// textblock starting with three backticks into a code block.
function codeBlockRule(nodeType) {
    return (0, prosemirror_inputrules_1.textblockTypeInputRule)(/^```$/, nodeType);
}
