"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.linkPasteEnhance = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const selection_1 = require("../../../utils/selection");
const utils_1 = require("../../behavior/Clipboard/utils");
const index_1 = require("./index");
function linkPasteEnhance({ parser }) {
    return new prosemirror_state_1.Plugin({
        props: {
            handleDOMEvents: {
                paste(view, e) {
                    const { state, dispatch } = view;
                    const sel = state.selection;
                    if (!(0, selection_1.isTextSelection)(sel))
                        return false;
                    const { $from, $to } = sel;
                    if ($from.pos !== $to.pos && $from.sameParent($to)) {
                        const url = getUrl(e.clipboardData, parser);
                        if (url) {
                            const tr = state.tr.addMark($from.pos, $to.pos, (0, index_1.linkType)(state.schema).create({
                                [index_1.LinkAttr.Href]: url,
                            }));
                            dispatch(tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, $to.pos)));
                            e.preventDefault();
                            return true;
                        }
                    }
                    return false;
                },
            },
        },
    });
}
exports.linkPasteEnhance = linkPasteEnhance;
function getUrl(data, parser) {
    var _a;
    if (!data || data.types.includes(utils_1.DataTransferType.Yfm))
        return null;
    if ((0, utils_1.isIosSafariShare)(data))
        return data.getData(utils_1.DataTransferType.UriList);
    // TODO: should we process HTML here?
    const text = data.getData(utils_1.DataTransferType.Text);
    const match = parser.matchLinks(text);
    if (((_a = match === null || match === void 0 ? void 0 : match[0]) === null || _a === void 0 ? void 0 : _a.raw) === text)
        return match[0].url;
    return null;
}
