"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reactNodeViewFactory = exports.ReactNodeView = exports.ReactNodeStopEventCn = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const react_dom_1 = require("react-dom");
const extensions_1 = require("../extensions");
exports.ReactNodeStopEventCn = 'prosemirror-stop-event';
class ReactNodeView {
    constructor(Component, opts) {
        const { getPos, node, serializer, view, options } = opts;
        this.node = node;
        this.dom = (options === null || options === void 0 ? void 0 : options.isInline)
            ? document.createElement('span')
            : document.createElement('div');
        this.dom.classList.add('react-node-wrapper');
        if (options === null || options === void 0 ? void 0 : options.reactNodeWrapperCn) {
            this.dom.classList.add(options === null || options === void 0 ? void 0 : options.reactNodeWrapperCn);
        }
        this.dom.contentEditable = 'false';
        this.serializer = serializer;
        this.view = view;
        this.getPos = getPos;
        this.renderItem = (0, extensions_1.getReactRendererFromState)(view.state).createItem(`${Component.displayName || this.node.type.name}-view`, () => (0, react_dom_1.createPortal)(react_1.default.createElement(Component, { dom: this.dom, view: this.view, updateAttributes: this.updateAttributes.bind(this), node: this.node, getPos: this.getPos.bind(this), serializer: this.serializer, extensionOptions: options === null || options === void 0 ? void 0 : options.extensionOptions }), this.dom));
    }
    update(node) {
        if (node.type !== this.node.type)
            return false;
        if (this.node === node)
            return true;
        this.node = node;
        this.renderItem.rerender();
        return true;
    }
    destroy() {
        this.renderItem.remove();
    }
    ignoreMutation() {
        return true;
    }
    updateAttributes(attributes) {
        const pos = this.getPos();
        const { tr } = this.view.state;
        tr.setNodeMarkup(pos, undefined, Object.assign(Object.assign({}, this.node.attrs), attributes));
        this.view.dispatch(tr);
    }
    stopEvent(e) {
        const target = e.target;
        const isInput = ['INPUT', 'BUTTON', 'SELECT', 'TEXTAREA'].includes(target.tagName);
        if (isInput ||
            (typeof target.className === 'string' &&
                target.className.includes(exports.ReactNodeStopEventCn))) {
            return true;
        }
        return false;
    }
}
exports.ReactNodeView = ReactNodeView;
const reactNodeViewFactory = (Component, options) => ({ serializer }) => (node, view, getPos) => new ReactNodeView(Component, { node, view, getPos, serializer, options });
exports.reactNodeViewFactory = reactNodeViewFactory;
