"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FlexToolbar = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const react_use_1 = require("react-use");
const icons_1 = require("@gravity-ui/icons");
const classname_1 = require("../classname");
const Toolbar_1 = require("./Toolbar");
const ToolbarListButton_1 = require("./ToolbarListButton");
const flexible_1 = require("./flexible");
const logger_1 = require("../logger");
const hooks_1 = require("../react-utils/hooks");
const types_1 = require("./types");
const b = (0, classname_1.cn)('flex-toolbar');
function FlexToolbar(props) {
    (0, hooks_1.useRenderTime)((time) => {
        logger_1.logger.metrics({
            component: 'toolbar',
            event: 'render',
            duration: time,
        });
    });
    const { data, className, hiddenActions } = props;
    const [ref, { width }] = (0, react_use_1.useMeasure)();
    const { data: items, dots } = react_1.default.useMemo(() => {
        const toolbarButtonIds = data.reduce((a, toolbarGroup) => {
            return [
                ...a,
                ...toolbarGroup
                    .map((toolbarButton) => {
                    if (toolbarButton.type === types_1.ToolbarDataType.ListButton) {
                        return toolbarButton.data.map((v) => v.id);
                    }
                    return toolbarButton.id;
                })
                    .flat(),
            ];
        }, []);
        // Finding only actions tha are not present in the main toolbar config
        const filteredHiddenAction = hiddenActions === null || hiddenActions === void 0 ? void 0 : hiddenActions.filter((a) => !toolbarButtonIds.includes(a.id));
        return (0, flexible_1.shrinkToolbarData)({
            data,
            availableWidth: width,
            hiddenActions: filteredHiddenAction,
        });
    }, [data, width, hiddenActions]);
    return (react_1.default.createElement("div", { ref: ref, className: b(null, [className]) },
        react_1.default.createElement("div", { className: b('container') },
            react_1.default.createElement(Toolbar_1.Toolbar, Object.assign({}, props, { data: items, className: b('bar') })),
            (dots === null || dots === void 0 ? void 0 : dots.length) && (react_1.default.createElement(ToolbarListButton_1.ToolbarListButton, { data: dots, icon: { data: icons_1.Ellipsis }, title: props.dotsTitle, editor: props.editor, focus: props.focus, onClick: props.onClick, className: b('dots'), alwaysActive: true })))));
}
exports.FlexToolbar = FlexToolbar;
