import { markTypeFactory } from '../../../../utils/schema';
export const linkMarkName = 'link';
export const linkType = markTypeFactory(linkMarkName);
export var LinkAttr;
(function (LinkAttr) {
    LinkAttr["Href"] = "href";
    LinkAttr["Title"] = "title";
    // tech attributes
    LinkAttr["IsPlaceholder"] = "is-placeholder";
    LinkAttr["RawLink"] = "raw-link";
})(LinkAttr || (LinkAttr = {}));
export const LinkSpecs = (builder) => {
    builder.addMark(linkMarkName, () => ({
        spec: {
            attrs: {
                [LinkAttr.Href]: {},
                [LinkAttr.Title]: { default: null },
                [LinkAttr.IsPlaceholder]: { default: false },
                [LinkAttr.RawLink]: { default: false },
            },
            inclusive: false,
            parseDOM: [
                {
                    tag: 'a[href]',
                    getAttrs(dom) {
                        return {
                            href: dom.getAttribute(LinkAttr.Href),
                            title: dom.getAttribute(LinkAttr.Title),
                        };
                    },
                },
            ],
            toDOM(node) {
                return ['a', node.attrs];
            },
        },
        toYfm: {
            open(state, mark, parent, index) {
                state.isAutolink = isPlainURL(mark, parent, index, 1);
                if (state.isAutolink) {
                    if (mark.attrs[LinkAttr.RawLink])
                        return '';
                    return '<';
                }
                return '[';
            },
            close(state, mark) {
                if (state.isAutolink) {
                    state.isAutolink = undefined;
                    if (mark.attrs[LinkAttr.RawLink])
                        return '';
                    return '>';
                }
                state.isAutolink = undefined;
                return ('](' +
                    mark.attrs[LinkAttr.Href] +
                    (mark.attrs[LinkAttr.Title]
                        ? ' ' + state.quote(mark.attrs[LinkAttr.Title])
                        : '') +
                    ')');
            },
        },
        fromYfm: {
            tokenSpec: {
                name: linkMarkName,
                type: 'mark',
                getAttrs: (tok) => ({
                    href: tok.attrGet('href'),
                    title: tok.attrGet('title') || null,
                }),
            },
        },
    }));
};
function isPlainURL(link, parent, index, side) {
    if (link.attrs.title || !/^\w+:/.test(link.attrs[LinkAttr.Href]))
        return false;
    const content = parent.child(index + (side < 0 ? -1 : 0));
    if (!content.isText ||
        content.text !== link.attrs[LinkAttr.Href] ||
        content.marks[content.marks.length - 1] !== link)
        return false;
    if (index === (side < 0 ? 1 : parent.childCount - 1))
        return true;
    const next = parent.child(index + (side < 0 ? -2 : 1));
    return !link.isInSet(next.marks);
}
