import { Plugin } from 'prosemirror-state';
import type { Node } from 'prosemirror-model';
import { NodeView } from 'prosemirror-view';
import type { ReactRenderer, RendererItem } from '../../../extensions/behavior/ReactRenderer';
import 'katex/dist/katex.min.css';
import './view-and-edit.css';
export declare abstract class MathNodeView implements NodeView {
    dom: HTMLElement;
    contentDOM: HTMLElement;
    protected node: Node;
    protected texContent: string;
    protected mathViewDOM: HTMLElement;
    protected mathHintContainerDOM: HTMLElement;
    protected hintRendererItem: RendererItem;
    protected readonly reactRenderer: ReactRenderer;
    abstract createDOM(): Record<'dom' | 'contentDOM' | 'mathViewDOM' | 'mathHintContainerDOM', HTMLElement> & {
        hintRendererItem: RendererItem;
    };
    abstract isDisplayMode(): boolean;
    constructor(node: Node, reactRenderer: ReactRenderer);
    update(node: Node): boolean;
    ignoreMutation(mutation: MutationRecord): boolean;
    destroy(): void;
    protected renderKatex(): void;
    protected updateErrorView(isError: boolean): void;
    protected getTexContent(): string;
}
export declare class MathInlineNodeView extends MathNodeView {
    destroy(): void;
    isDisplayMode(): boolean;
    createDOM(): {
        dom: HTMLSpanElement;
        contentDOM: HTMLSpanElement;
        mathViewDOM: HTMLSpanElement;
        mathHintContainerDOM: HTMLDivElement;
        hintRendererItem: RendererItem;
    };
    protected createMathInlineDOM(): {
        container: HTMLSpanElement;
        content: HTMLSpanElement;
    };
}
export declare class MathBlockNodeView extends MathNodeView {
    isDisplayMode(): boolean;
    createDOM(): {
        dom: HTMLDivElement;
        contentDOM: HTMLDivElement;
        mathViewDOM: HTMLDivElement;
        mathHintContainerDOM: HTMLDivElement;
        hintRendererItem: RendererItem;
    };
}
export declare const mathViewAndEditPlugin: ({ reactRenderer }: {
    reactRenderer: ReactRenderer;
}) => Plugin<any>;
