import { Node as PmNode } from 'prosemirror-model';
import { findChildren, findParentNode, findParentNodeClosestToPos, } from 'prosemirror-utils';
import { isTextSelection } from '../utils/selection';
import { TableRole } from './const';
export const isTableNode = (node) => node.type.spec.tableRole === TableRole.Table;
export const isTableBodyNode = (node) => node.type.spec.tableRole === TableRole.Body;
export const isTableRowNode = (node) => node.type.spec.tableRole === TableRole.Row;
export const isTableCellNode = (node) => node.type.spec.tableRole === TableRole.Cell;
export const findParentTable = findParentNode(isTableNode);
export const findParentTableBody = findParentNode(isTableBodyNode);
export const findParentTableRow = findParentNode(isTableRowNode);
export const findParentTableCell = findParentNode(isTableCellNode);
export const findParentTableFromPos = ($pos) => findParentNodeClosestToPos($pos, isTableNode);
export const findParentTableBodyFromPos = ($pos) => findParentNodeClosestToPos($pos, isTableBodyNode);
export const findParentTableRowFromPos = ($pos) => findParentNodeClosestToPos($pos, isTableRowNode);
export const findParentTableCellFromPos = ($pos) => findParentNodeClosestToPos($pos, isTableCellNode);
export const findChildTableRows = (node) => findChildren(node, isTableRowNode);
export const findChildTableCells = (node) => findChildren(node, isTableCellNode);
export const findChildTableBody = (node) => findChildren(node, isTableBodyNode);
export const getTableDimensions = (node) => {
    var _a, _b, _c, _d, _e, _f;
    let rows, cols;
    if (node instanceof PmNode) {
        rows = (_a = node.firstChild) === null || _a === void 0 ? void 0 : _a.childCount;
        cols = (_c = (_b = node.firstChild) === null || _b === void 0 ? void 0 : _b.firstChild) === null || _c === void 0 ? void 0 : _c.childCount;
    }
    else {
        rows = (_d = node.firstChild) === null || _d === void 0 ? void 0 : _d.childNodes.length;
        cols = (_f = (_e = node.firstChild) === null || _e === void 0 ? void 0 : _e.firstChild) === null || _f === void 0 ? void 0 : _f.childNodes.length;
    }
    return { rows: rows || 1, cols: cols || 1 };
};
export function atEndOfCell(view, dir) {
    if (!isTextSelection(view.state.selection))
        return null;
    const { $head } = view.state.selection;
    for (let d = $head.depth - 1; d >= 0; d--) {
        const parent = $head.node(d), index = dir < 0 ? $head.index(d) : $head.indexAfter(d);
        if (index !== (dir < 0 ? 0 : parent.childCount))
            return null;
        if (parent.type.spec.tableRole === TableRole.Cell) {
            const cellPos = $head.before(d);
            const dirStr = dir > 0 ? 'down' : 'up';
            return view.endOfTextblock(dirStr) ? cellPos : null;
        }
    }
    return null;
}
